# Generated by Django 2.2.18 on 2021-02-24 13:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("terra_opp", "0005_auto_20210224_1243"),
    ]

    operations = [
        migrations.AlterField(
            model_name="campaign",
            name="start_date",
            field=models.DateField(verbose_name="Start date"),
        ),
    ]
