# Generated by Django 3.1.7 on 2021-04-12 10:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("terra_opp", "0009_picture_campaign"),
    ]

    operations = [
        migrations.AddField(
            model_name="picture",
            name="identifier",
            field=models.PositiveIntegerField(null=True, verbose_name="Identifier"),
        ),
    ]
