# Generated by Django 2.2.18 on 2021-04-27 12:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("terra_opp", "0013_rename_picture_newidentifier"),
    ]

    operations = [
        migrations.AddField(
            model_name="theme",
            name="category",
            field=models.CharField(default="", max_length=100, verbose_name="Category"),
        ),
    ]
