# Generated by Django 3.1.7 on 2021-03-10 16:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("terra_opp", "0003_auto_20200630_1315"),
    ]

    operations = [
        migrations.AddField(
            model_name="viewpoint",
            name="active",
            field=models.BooleanField(default=True, verbose_name="Active"),
        ),
    ]
