# Generated by Django 2.2.18 on 2021-03-25 10:22

from django.db import migrations, models
import terra_opp.models
import versatileimagefield.fields


class Migration(migrations.Migration):

    dependencies = [
        ("terra_opp", "0006_auto_20210224_1333"),
    ]

    operations = [
        migrations.RenameField(
            model_name="picture",
            old_name="state",
            new_name="old_state",
        ),
        migrations.AlterModelOptions(
            name="campaign",
            options={"ordering": ["-start_date", "-created_at"]},
        ),
        migrations.AddField(
            model_name="campaign",
            name="state",
            field=models.CharField(
                choices=[
                    ("draft", "Draft"),
                    ("started", "Started"),
                    ("closed", "Closed"),
                ],
                default="draft",
                max_length=10,
                verbose_name="State",
            ),
        ),
        migrations.AddField(
            model_name="picture",
            name="state",
            field=models.CharField(
                choices=[
                    ("draft", "Draft"),
                    ("submited", "Submited"),
                    ("accepted", "Accepted"),
                    ("refused", "Refused"),
                ],
                default="draft",
                max_length=10,
                verbose_name="State",
            ),
        ),
        migrations.AlterField(
            model_name="campaign",
            name="label",
            field=models.CharField(max_length=100, verbose_name="Label"),
        ),
        migrations.AlterField(
            model_name="picture",
            name="file",
            field=versatileimagefield.fields.VersatileImageField(
                upload_to=terra_opp.models.image_upload_to, verbose_name="File"
            ),
        ),
    ]
